import java.util.*;

/**
 * This class represents an customer list by means of an array of customers. 
 * 
 * @author Bernd J. Krmer
 * @version 22.12.2003
 */
public class CustomerList
{
    private Customer [] customers;
    // indicate the place in the list where the next customer can be added.
    private int here = 0;
    
    /**
     * Create a Customer list with a maximum number of customers.
     * @param  maxNumberOfCustomers   the capacity of the list
     */
    public CustomerList(int maxNumberOfCustomers)
    {
        customers = new Customer [maxNumberOfCustomers]; 
    }

    /**
     * Add a customer to this list.
     * @param  newCustomer   the Customer object to be added
     */
    public boolean tryAddCustomer(Customer newCustomer)
    {
        int index;
        if (isInList(newCustomer))
        {
            System.out.println("Customer: " + newCustomer.getName() + " is already listed!");
        }
        if ((index = checkIfEmptySlot()) != -1) 
        {
            customers[index] = newCustomer;
            return true;
        } else 
        {
            if (here < customers.length)
            {
                customers[here++] = newCustomer;
                return true;
            } else
            {
                System.out.println("The customer list is full;" 
                       + " customer " + newCustomer.getName() 
                       + " cannot be added.");
                return false;
            }
        }
    }

        /**
     * Try to remove a customer from this list.
     * @param  thisCustomer   the Customer object to be added
     * @return true if thisCustomer is in list; fals otherwise
     */
    public boolean tryRemoveCustomer(Customer thisCustomer)
    {
        int index = getIndexOfCustomer(thisCustomer);
        if (index == -1) 
        {
            System.out.println("Customer " 
                       + thisCustomer.getName() 
                       + " is  not contained in this list.");
            return false;
        }
        else 
        {
            customers[index] = null; 
            return true;
        }
    }
    
   /**
     * Get the index of a customer in the customer list.
     * @return     the index of the customer if in list; otherwise -1.
     */
    private int getIndexOfCustomer(Customer thisCustomer)
    {
        int index = -1;         // not in list
        for (int i = 0; i < customers.length && i < here; i++)
        {
            if (customers[i] == thisCustomer)
            {
                index = i;      // customer found
                break;          // break out of loop
            }
        }
        return index;
    }
    
    /**
     * Check if there is an empty slot in the list.
     * @return     the index of the empty slot; otherwise -1.
     */
    private int checkIfEmptySlot()
    {
        return getIndexOfCustomer(null);
    }
    
    /**
     * Return the number of customers currently in customer list.
     * @return     the number of customers currently in customer list.
     */
    public int numberOfCustomers()
    {
        return here;
    }
    
    /**
     * Check if a customer is already in the list
     * 
     * @param  c   the Customer to be identified
     * @return     true if found, fals otherwise 
     */
    public boolean isInList(Customer c)
    {
        return getIndexOfCustomer(c) != -1;
    }
    
    /**
     * Print out a customer list to the standard terminal.
     */
    public void printList()
    {
        for (int i = 0; i < customers.length && i < here; i++)
        {
            if (customers[i] != null)
            {
                customers[i].print();
            }
        }
        System.out.println("Number of customers: " + numberOfCustomers());
    }
}
